/*
 * Created using SharpDevelop.
 * User: Evan Dudzik - eegeek.net
 * Date: 2/14/2008
 */
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

using DWORD = System.UInt32;
using HANDLE = System.UInt32;

namespace PICUSB
{
	/// <summary>
    /// Generic interface to a USB PIC using normal VID/PID
    /// </summary>
	public class usb_interface
	{
		/// <summary>
		/// Microchip-provided functions imported from DLL
		/// </summary>
		protected class MPUSB
		{
			/// <summary>
			/// A file handle returned by the Microchip functions is invalid if it matches this value.
			/// </summary>
			public const HANDLE INVALID_HANDLE_VALUE = 0xFFFFFFFF;
			/// <summary>
			/// Success.
			/// </summary>
	      	public const DWORD SUCCESS = 1;
	      	/// <summary>
	      	/// Failure.
	      	/// </summary>
	      	public const DWORD FAIL    = 0;
	      	
	      	/// <summary>
	      	/// Valid values representing transfer direction for pipes opened to USB endpoints.
	      	/// These directions are relative to the PC.
	      	/// </summary>
	      	public enum PipeDir : uint
		    {
	      		/// <summary>
	      		/// For transferring to an OUT endpoint on the PIC.
	      		/// </summary>
		       Write = 0,
		       /// <summary>
		       /// For reading from an IN endpoint on the PIC.
		       /// </summary>
		       Read = 1
		    }
	      	
	      	/// <summary>
	      	/// Gets the version number of the MPUSBAPI.DLL file.
	      	/// </summary>
	      	/// <returns>Version number of MPUSBAPI.DLL</returns>
			[DllImport("mpusbapi.dll",EntryPoint="_MPUSBGetDLLVersion")]
			public static extern DWORD MPUSBGetDLLVersion();
			
			/// <summary>
			/// Scans USB bus to determine number of devices connected with a given PID/VID.
			/// </summary>
			/// <param name="pVID_PID">PID/VID in question</param>
			/// <returns>Number of devices found</returns>
			[DllImport("mpusbapi.dll", EntryPoint = "_MPUSBGetDeviceCount")]
			public static extern DWORD GetDeviceCount([MarshalAs(UnmanagedType.LPStr)] string pVID_PID);
			
			/// <summary>
			/// Opens a pipe to a USB endpoint.
			/// </summary>
			/// <param name="instance">Index of PIC to connect to.  Use zero if only one PIC is connected.</param>
			/// <param name="pVID_PID">VID/PID of PIC</param>
			/// <param name="pEP">String representing endpoint to connect to</param>
			/// <param name="dwDir">Direction of transfer</param>
			/// <param name="dwReserved">0</param>
			/// <returns>Handle of opened pipe (or INVALID handle if failed)</returns>
			[DllImport("mpusbapi.dll", EntryPoint = "_MPUSBOpen", SetLastError = true)]
			public static extern DWORD Open   ( DWORD   instance,
			[MarshalAs(UnmanagedType.LPStr)] string  pVID_PID,
			[MarshalAs(UnmanagedType.LPStr)] string  pEP,
			   [MarshalAs(UnmanagedType.U4)] PipeDir dwDir,
			                                  DWORD   dwReserved );
			
			/// <summary>
			/// Read data from a PIC endpoint
			/// </summary>
			/// <param name="handle">Handle of pipe to read from</param>
			/// <param name="pData">Array to accept incoming data</param>
			/// <param name="dwLen">Expected length of received data(?)</param>
			/// <param name="pLength">Actual length of received data(?)</param>
			/// <param name="dwMilliseconds">Timeout, in milliseconds</param>
			/// <returns>1=success</returns>
			[DllImport("mpusbapi.dll", EntryPoint = "_MPUSBRead", SetLastError = true)]
			public static extern DWORD Read   ( HANDLE handle,
			[MarshalAs(UnmanagedType.LPArray)] byte[] pData,
			                                  DWORD  dwLen,
			                              out DWORD  pLength,
			                                  DWORD  dwMilliseconds );
			
			/// <summary>
			/// Write data to a PIC endpoint
			/// </summary>
			/// <param name="handle">Handle of pipe to write to</param>
			/// <param name="pData">Data to send</param>
			/// <param name="dwLen">Length of data array</param>
			/// <param name="pLength">(?)</param>
			/// <param name="dwMilliseconds">Timeout, in milliseconds</param>
			/// <returns>1=success</returns>
			[DllImport("mpusbapi.dll", EntryPoint = "_MPUSBWrite", SetLastError = true)]
			public static extern DWORD Write  ( HANDLE handle,
			[MarshalAs(UnmanagedType.LPArray)] byte[] pData,
			                                  DWORD  dwLen,
			                              out DWORD  pLength,
			                                  DWORD  dwMilliseconds );
			
			/// <summary>
			/// Read from an asynchronous interrupt endpoint on a PIC
			/// </summary>
			/// <param name="handle">Handle of pipe to read from</param>
			/// <param name="pData">Array for incoming data</param>
			/// <param name="dwLen">Expected received data length(?)</param>
			/// <param name="pLength">Actual received data length(?)</param>
			/// <param name="dwMilliseconds">Timeout, in milliseconds</param>
			/// <returns>1=success</returns>
			[DllImport("mpusbapi.dll", EntryPoint = "_MPUSBReadInt")]
			public static extern DWORD ReadInt( HANDLE handle,
			[MarshalAs(UnmanagedType.LPArray)] byte[] pData,
			                                  DWORD  dwLen,
			                              out DWORD  pLength,
			                                  DWORD  dwMilliseconds );
			
			/// <summary>
			/// Close a pipe to a PIC endpoint
			/// </summary>
			/// <param name="handle">Handle of pipe to close</param>
			/// <returns>1=success</returns>
			[DllImport("mpusbapi.dll", EntryPoint = "_MPUSBClose")]
			public static extern bool Close   ( HANDLE handle );
			
			/// <summary>
			/// Gets the version of MPUSBAPI.DLL
			/// </summary>
			public static DWORD DLLVersion { get { return MPUSBGetDLLVersion(); } }
			/// <summary>
			/// Gets the version of MPUSBAPI.DLL, formatted.
			/// </summary>
			/// <param name="major">Major version number</param>
			/// <param name="minor">Minor version number</param>
			public static void GetDLLVersion(out ushort major, out ushort minor)
			{
				DWORD ver = DLLVersion;
				major = (ushort)(DLLVersion >> 16);
				minor = (ushort)(DLLVersion & 0x0000FFFF);
			}
		}
		
		#region  String Definitions of Pipes and VID_PID
        /// <summary>
        /// VID/PID for PIC running Microchip default USB bootloader firmware
        /// </summary>
        public string vid_pid_boot = "vid_04d8&pid_ff0b";
        /// <summary>
        /// VID/PID for PIC running generic firmware
        /// </summary>
        public string vid_pid_norm = "vid_04d8&pid_ff0b";

        /// <summary>
        /// OUT endpoint on PIC
        /// </summary>
		protected string out_pipe= "\\MCHP_EP1";
        /// <summary>
        /// IN endpoint on PIC (synchronous)
        /// </summary>
		protected string in_pipe= "\\MCHP_EP1";
        /*/// <summary>
        /// IN endpoint on PIC (asynchronous)
        /// </summary>
        protected string in_pipe_async = "\\MCHP_EP1_ASYNC";*/
		#endregion
        
        #region Endpoint Pipes
		/// <summary>
        /// OUT endpoint pipe
        /// </summary>
		protected uint myOutPipe;
        /// <summary>
        /// IN endpoint pipe (synchronous)
        /// </summary>
		protected uint myInPipe;

        /// <summary>
        /// Represents whether pipes have been opened
        /// </summary>
        protected bool prvPipesOpen;
        /// <summary>
        /// Represents whether pipes have been opened
        /// </summary>
        #endregion
        
        public bool PipesOpen
        {   get { return prvPipesOpen; }	set { }	}

        /// <summary>
        /// Commands recognized by generic firmware.  May be overridden for custom commands.
        /// </summary>
        public enum Commands : byte
        {
            /// <summary>
            /// Turn LED 3 or 4 on demo board on or off.
            /// </summary>
            UPDATE_LED = 0x32,
            /// <summary>
            /// Instructs PIC to reset
            /// </summary>
            RESET = 0xFF
        }

        /// <summary>
        /// Opens USB pipes to the PIC using the normal VID/PID
        /// </summary>
		protected virtual void OpenPipes()
		{
			DWORD selection=0; // Selects the device to connect to
			myOutPipe = MPUSB.Open(selection,vid_pid_norm,out_pipe,MPUSB.PipeDir.Write,0);
			myInPipe = MPUSB.Open(selection,vid_pid_norm,in_pipe,MPUSB.PipeDir.Read,0);
            prvPipesOpen = true;
		}

        /// <summary>
        /// Constructor
        /// </summary>
        public usb_interface()
        {
            prvPipesOpen = false;
        }

        /// <summary>
        /// Closes all open pipes.  Should be called if a different instance of the class needs access
        /// </summary>
		public virtual void ClosePipes()
		{
			MPUSB.Close(myOutPipe);
			MPUSB.Close(myInPipe);
            prvPipesOpen = false;
		}

        /// <summary>
        /// Writes one packet to PIC, receives response packet.
        /// </summary>
        /// <param name="SendData">Pointer to bytes to be sent</param>
        /// <param name="SendLength">Number of bytes to be sent</param>
        /// <param name="ReceiveData">Pointer to location for received data</param>
        /// <param name="ReceiveLength">Expected number of received bytes</param>
        /// <returns>1=Success, 2=Incorrect RX Length, 3=Read Failed</returns>
        protected DWORD SendReceivePacket(byte[] SendData, DWORD SendLength, byte[] ReceiveData, DWORD ReceiveLength)
		{
            if (prvPipesOpen==false)
                OpenPipes();
            uint SendDelay = 1000;
            uint ReceiveDelay = 1000;
            DWORD rval = 0;
            DWORD SentDataLength;
            DWORD ExpectedReceiveLength = ReceiveLength;
            if (MPUSB.Write(myOutPipe, SendData, SendLength, out SentDataLength, SendDelay) == 1)
            {

                if (MPUSB.Read(myInPipe, ReceiveData, ExpectedReceiveLength, out ReceiveLength, ReceiveDelay) == 1)
                {
                    if (ReceiveLength == ExpectedReceiveLength)
                    {
                        rval = 1;   // Success!
                    }
                    else if (ReceiveLength < ExpectedReceiveLength)
                    {
                        rval = 2;   // Partially failed, incorrect receive length
                    }
                }
                else
                {
                    rval = 3;   //Read Failed
                }
            }
            return rval;
		}

        /// <summary>
        /// Version number of mpusbapi.dll
        /// </summary>
        public DWORD DLLVersion{	get{	return MPUSB.DLLVersion;	}	}
        
        /// <summary>
        /// Scans the USB bus for connected devices with matching VID/PID
        /// </summary>
        /// <param name="Vid_Pid">VID/PID to scan for</param>
        /// <returns>Number of connected devices</returns>
		public DWORD GetDeviceCount(string Vid_Pid)
		{
			return MPUSB.GetDeviceCount(Vid_Pid);
		}

        /// <summary>
        /// Simple method for sending/receiving USB data from the PIC.
        /// </summary>
        /// <param name="Command">Command byte</param>
        /// <param name="rxlength">Expected length of received packet</param>
        /// <param name="data">Array of bytes to send after command - pass null if not applicable</param>
        /// <param name="dataout">Array for received bytes</param>
        /// <returns>1=Success, 2=Incorrect RX Length, 3=Read Failed</returns>
        public uint EasyCommand(byte Command, int rxlength, byte[] data, out byte[] dataout)
        {            
            byte[] send_buf = new byte[64];		//Buffer for sent data
            byte[] receive_buf = new byte[64];	//Buffer for received data
            
            send_buf[0] = Command;
            if(data==null){ data = new byte[]{};	} //Empty array instead of null
            else
            {	//Copy data into send_buffer after command byte
	            for (int i = 0; i < data.Length; i++)
	            { send_buf[i + 1] = data[i]; }
            }
	        uint rval = SendReceivePacket(send_buf, 1 + (uint)data.Length, receive_buf, (DWORD)rxlength);
            if (rval != 1) //If there was an error
            { dataout = null; return rval; }
            dataout = new byte[rxlength]; 		//Create output data array
            for (int i = 0; i < rxlength; i++)	//Copy data into it
            { dataout[i] = receive_buf[i]; }
            return 1;
        }


        /// <summary>
        /// Anzahl Bytes zu senden, Anzahl Bytes welche empfangen werden, 
        /// </summary>
        /// <param name="txlength"></param>
        /// <param name="rxlength"></param>
        /// <param name="data"></param>
        /// <param name="dataout"></param>
        /// <returns></returns>
        public uint EasyCommand(int txlength, int rxlength, byte[] data, out byte[] dataout)
        {
            byte[] send_buf = new byte[20];		//Buffer for sent data
            byte[] receive_buf = new byte[20];	//Buffer for received data

            if (data == null) { data = new byte[] { }; } //Empty array instead of null
            else
            {	//Copy data into send_buffer after command byte
                for (int i = 0; i < txlength; i++)
                { send_buf[i] = data[i]; }
            }
            uint rval = SendReceivePacket(send_buf, (uint)txlength, receive_buf, (DWORD)rxlength);
            if (rval != 1) //If there was an error
            { dataout = null; return rval; }
            dataout = new byte[rxlength]; 		//Create output data array
            for (int i = 0; i < rxlength; i++)	//Copy data into it
            { dataout[i] = receive_buf[i]; }
            return 1;
        }

	}
}
